#!/usr/bin/python
#
#		X T E N S I O N   S W I T C H B O T   P L U G I N
#
#	
#	version 1.0 james@sentman.com
#	support for the switchbot from http://www.switch-bot.com
#	
# 	no support for the security settings. If you have turned on the pairing requirement in the
#	switchbot app you'll have to turn that off in order for this to talk to the device.
#
#	version 1 cannot read the battery level or make other changes to the settings of the device
#	however if you make changes to things such as the travel distance or press time those are
#	saved in the device and will still be active when you control the device from XTension
#




import sys
import os
import subprocess
import threading

#
# Importing Necessary XTension Connection Modules
#

from os.path import expanduser
userHomePath = expanduser( '~')
del expanduser

pluginLocationFile = open( userHomePath + '/Documents/XTension Support/.python_include_path.txt', 'r')

if pluginLocationFile.mode == 'r':
	pluginLocationPath = pluginLocationFile.read()
	pluginLocationFile.close
else:
	print( "Error opening the XTension Plugin Includes path file, cannot continue")
	sys.exit()
	

pluginLocationPath += "/current"
	
sys.path.append( pluginLocationPath )

from xtension_plugin.xtension_constants import *
from xtension_plugin.xtension_plugin import *



#
#  P L U G I N   S P E C I F I C   C O D E   S T A R T S   B E L O W
#
#

pluginVersion			= '1.0' # used for initial log in XTension


#
# 	Device Tags
#

tagSwitchbot			= 'xt.switchbot'

#
#	Interface Keys
#
keySwitchbotBehavior	= 'xt.switchbot.behavior'

#
# the keySwitchbotBehavior will contain one or the other of these
# as set by the popup in the interface
#
behaviorPress			= 'press'
behaviorToggle			= 'toggle'

keySwitchbotCount		= 'xt.switchbot.count' 		# the value entered to search for when performing a scan
keySwitchbotTimeout		= 'xt.switchbot.timeout' 	# the timeout value entered in the edit interface dialog



#
# Script Handler Names
#

#	scan for new or existing switchbots
#
#	tell xInterface "name" to scanForSwitchbots( count, timeout)
#
#	pass a number of switchbots to look for, it will stop scanning then
#	that number have been found, up until the timeout period.
#	passing a number more than the number of possibly available switchbots
#	will cause the interafce to be unavailable for the timeout period.
#
# this is the script one with positional params
handlerScan				= 'scanForSwitchbots'	

# this is the handler when click the button on the interface
interfaceScan			= 'intScanForSwitchbots'
									


#
# switchbot path, won't run it just in our same directory for whatever reason
# need a full path, but it could be different. It seems that sys.path[0] always contains
# the path to our parent folder, but thats not for certain I don't think. This might
# not always work.
#
helperPath = sys.path[0] + "/switchbot"
print( "helper path is: " + helperPath)




#
# Make Unit Subclass
#
#	creates an xtUnit subclass for the device types we have specified 
# 	in this case there is only the one, but there might be more in the future
#

def makeUnitSubclass( theData):
	newUnitTag = theData.get( xtUnitKeyTag)
	
	if newUnitTag == tagSwitchbot:
		return xtSwitchbotUnit( theData)
	else:
		# if it's not a unit we recognize then just return the default
		# this should not happen as we only have one unit type
		# but it's possible to transfer non-switchbot units to this interface
		return xtUnit( theData)
		

#
#	xtSwitchBotUnit
#
#	the subclass of xtUnit that handles any commands from XTension meant for a switchbot unit
#

class xtSwitchbotUnit( xtUnit):
	def __init__( self, theData):
		# always pass the data up to the super class constructor before continuing!
		xtUnit.__init__( self, theData)
		
		# add our command handlers for on/off control from XTension
		self.addCommandHandler( xtCommandOn, self.handleOnCommand)
		self.addCommandHandler( xtCommandOff, self.handleOffCommand)
		
		
	
	def handleOnCommand( self, theCommand):
	
		# get the proper on behavior with a default to press
		thisBehavior = self.data.get( keySwitchbotBehavior, behaviorPress)
		
		if thisBehavior == behaviorPress:
		
			if XTension.debugMode:
				XTension.writeLog( "sending press")
			
			#self.doPress()
			self.pressThread = Thread( target=self.doPress, args=())
			self.pressThread.start()

			# and now we immediately send an off for the XTension unit as a press doesn't
			# leave the switch in the On position
			

			
		else:
		
			if XTension.debugMode:
				XTension.writeLog( "toggle on")
			
			self.toggleThread = Thread( target=self.doToggle, args=( True,))
			self.toggleThread.start()
			
			#self.doToggle( True)
			
			
			
			
	def handleOffCommand( self, theCommand):
	
		thisBehavior = self.data.get( keySwitchbotBehavior, behaviorPress)
		
		if thisBehavior == behaviorPress:
			XTension.writeLog( "Off command ignored for Press behavior switchbot unit")
			return
			
		if XTension.debugMode:
			XTension.writeLog( "toggle off")
			
		#self.doToggle( False)
		
		self.toggleThread = Thread( target=self.doToggle, args=( False,))
		self.toggleThread.start()
		
		
		
		
	#
	#
	# here are the handlers that actually execute the shell command to talk to the switchbot
	#
	
	def doPress( self):
		
		try:
			# the unique id of the switchbot is our address
			
			reply = subprocess.check_output( [helperPath, "press", "--uuid", self.address])
			
			if reply != None and reply != '':
				XTension.writeLog( "switchbot reply: " + reply)
			
		except subprocess.CalledProcessError, e:
			theMessage = e.output
			if theMessage == None:
				theMessage = "no message"
				
			XTension.writeLog( "Error controlling switchbot (" + str( e.returncode) + ") " + theMessage, xtLogRed)
		
		
		x = XTCommand()
		x.set( xtKeyCommand, xtCommandOff)
		x.set( xtKeyAddress, self.address)
		x.set( xtKeyTag, self.tag)
		XTension.sendCommand( x)

		
		
			
	def doToggle( self, direction):
	
		if direction:
			theDirection = "toggleOn"
		else:
			theDirection = "toggleOff"
			
	
		try:
			reply = subprocess.check_output( [helperPath, theDirection, "--uuid", self.address])
			
			# there shouldn't be any output normally
			if reply != None and reply != '':
				XTension.writeLog( "switchbot reply: " + reply)
		
		except subprocess.CalledProcessError, e:
		
			theMessage = e.output
			if theMessage == None:
				theMessage = "no output"
				
			XTension.writeLog( "Error controlling switchbot (" + str( e.returncode) + ") " + theMessage, xtLogRed)
	
	
	
#
#   SCAN FOR SWITCHBOTS
#

def intScanForSwitchbots( commandName, positional, data):
	XTension.writeLog( "begin scan for switchbots...")
	scanThread = Thread( target=threadedScanWithData, args=(data,))
	scanThread.start()
			



def threadedScanWithData( data):
	try:
		reply = subprocess.check_output( [helperPath, "scan", "--count", data.get( keySwitchbotCount, "1"), "--timeout", data.get( keySwitchbotTimeout, "30")])
		
	except subprocess.CalledProcessError, e:
		theMessage = e.output
		if theMessage == None:
			theMessage = "no output"
			
		XTension.writeLog( "Error scanning for switchbots (" + str( e.returncode) + ") " + theMessage, xtLogRed)
		return
		
	# the reply should now have a list of available switchbots one on each line
	
	if XTension.debugMode:
		XTension.writeLog( "full output: " + reply)
	
	lines = reply.split( '/r')
	if XTension.debugMode:
		XTension.writeLog( "splits to " + str( len( lines)) + " lines")
	
	if len( lines) == 0 or lines[0].strip() == '':
		XTension.writeLog( "no switchbots found", xtLogRed)
		return
		
	
	for thisAddress in lines:
		thisAddress = thisAddress.strip() # just validate no extra line endings or spaces in them
			
		if XTension.debugMode:
			XTension.writeLog( "(" + thisAddress + ")")

		if thisAddress != '':
			XTension.writeLog( "found SwitchBot with uuid of: " + thisAddress)
	
			x = XTCommand()
			x.set( xtKeyCommand, xtCommandNewUnit)
			x.set( xtKeyAddress, thisAddress)
			x.set( xtKeyTag, tagSwitchbot)
			x.set( xtKeyName, "Switchbot (" + thisAddress + ")")
			x.set( xtKeyList, "Switchbots")
	
			XTension.sendCommand( x)
		
	XTension.writeLog( "done searching for switchbots")


#
#
#
#	M A I N   
#
#
#

	

XTension = cXTension()

# insert the makeUnitSubclass handler so that we will create the proper unit for the type
XTension.onMakeNewUnit = makeUnitSubclass

# uncomment to have debug mode on at startup
#XTension.debugMode = True

# the handler for pressing the button on the edit interface button
XTension.addScriptHandler( interfaceScan, intScanForSwitchbots)

# the handler for scanning from a script command
#XTension.addScriptHandler( handlerScan, scanForSwitchbots)

XTension.startup()



if not XTension.ready:
	print( "Failed to make connection to XTension. Shutting down\r")
	sys.exit()
else:
	XTension.writeLog( "XTension Switchbot Plugin v" + pluginVersion + " starting up")

